/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GRAPHICPRIMITIVE_H
#define GRAPHICPRIMITIVE_H

#include "matrix3x3.h"
#include <QPainter>


/// This class is the base of all primitives
/**
	This class is the base of all the figures we implement.
*/

class GraphicPrimitive
{
public:

	GraphicPrimitive();
	/// Draw the primitive
	/**
		As this function is virtual, all the descending classes must implement it.
	*/
	virtual void draw(Matrix3X3 * incomingMatrix, QPainter * painter) = 0 ;

	/// Get the x displacement
	double getXDisplacement();

	/// Get the y displacement
	double getYDisplacement();

	/// Get the rotation angle
	double getAngleOfRotation();

	/// Get the x scale
	double getXScale();

	/// Get the y scale
	double getYScale();

	/// Set the displacement in the x coordinate
	void setXDisplacement(double displacement);

	/// Set the displacement in the y coordinate
	void setYDisplacement(double displacement);

	/// Set the angle of rotation
	/**
		TODO: check wheter we use degrees or radians and document it.
	*/
	void setAngleOfRotation(double theta);

	/// Set the scalement in x
	void setXScale(double scale);

	/// Set the scalement in y
	void setYScale(double scale);

protected:
	double xDisplacement;
	double yDisplacement;
	double angleOfRotation;
	double xScale;
	double yScale;
};

#endif // GraphicPrimitive



