/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "matrix3x3.h"
#include <QString>
#include <QChar>
#include <QPointF>
#include <QDebug>

Matrix3X3::Matrix3X3()
{
	matrix[0][0] = 1.0;
	matrix[0][1] = 0.0;
	matrix[0][2] = 0.0;
	matrix[1][0] = 0.0;
	matrix[1][1] = 1.0;
	matrix[1][2] = 0.0;
	matrix[2][0] = 0.0;
	matrix[2][1] = 0.0;
	matrix[2][2] = 1.0;
}

void Matrix3X3::setValue(int i, int j, double value)
{
	if((i < 3) && (j < 3))
		matrix[i][j] = value;
}

Matrix3X3 * Matrix3X3::multiplyToTheRight(Matrix3X3 rightMatrix)
{
	double temp[3][3];
	Matrix3X3 * result = new Matrix3X3();

	for(int i = 0; i < 3; i++)
	{
		temp[i][0] = matrix[i][0] * rightMatrix.getComponent(0, 0) +
		             matrix[i][1] * rightMatrix.getComponent(1, 0) +
		             matrix[i][2] * rightMatrix.getComponent(2, 0);
		temp[i][1] = matrix[i][0] * rightMatrix.getComponent(0, 1) +
		             matrix[i][1] * rightMatrix.getComponent(1, 1) +
		             matrix[i][2] * rightMatrix.getComponent(2, 1);
		temp[i][2] = matrix[i][0] * rightMatrix.getComponent(0, 2) +
		             matrix[i][1] * rightMatrix.getComponent(1, 2) +
		             matrix[i][2] * rightMatrix.getComponent(2, 2);
	}

	for(int i=0; i<3; i++)
		for(int j=0; j<3; j++)
			result->setValue(i, j, temp[i][j]);

	return result;
}

QPointF Matrix3X3::multiplyPoint(QPointF point)
{
	QPointF temp;
	// Note that we asume w = 1 (homogeneous coordinates)
	temp.rx() = matrix[0][0] * point.x() + matrix[0][1] * point.y() + matrix[0][2];
	temp.ry() = matrix[1][0] * point.x() + matrix[1][1] * point.y() + matrix[1][2];

	return temp;
}

double & Matrix3X3::getComponent(const int & row, const int & column)
{
	return matrix[row][column];
}

QString Matrix3X3::getMatrixString()
{
	QString string;
	string.append("%1 %2 %3\n").arg(matrix[0][0],5,'g',3,QChar(' '))
	                           .arg(matrix[0][1],5,'g',3,QChar(' '))
	                           .arg(matrix[0][2],5,'g',3,QChar(' '));
	string.append("%1 %2 %3\n").arg(matrix[1][0],5,'g',3,QChar(' '))
	                           .arg(matrix[1][1],5,'g',3,QChar(' '))
	                           .arg(matrix[1][2],5,'g',3,QChar(' '));
	string.append("%1 %2 %3").arg(matrix[2][0],5,'g',3,QChar(' '))
	                         .arg(matrix[2][1],5,'g',3,QChar(' '))
	                         .arg(matrix[2][2],5,'g',3,QChar(' '));
	return string;
}

Matrix3X3 & Matrix3X3::operator=(Matrix3X3 & rightMatrix)
{
	if(this == &rightMatrix)
		return *this;

	for(int i=0; i < 3; i++)
		for(int j=0; j < 3; j++)
			matrix[i][j] = rightMatrix.getComponent(i, j);
	return *this;
}


